@@ -202,25 +202,31 @@ is.plotly <- function(x) {
 #'
 #' @param heatmap_layers ggplot object (eg, theme_bw()) to be added to
 #'  the heatmap before conversion to a plotly object.
-#'
+#' @param side_color_layers Layers to be added to side color plots, similar to
+#' 	heatmap_layers.
 #' @param branches_lwd numeric (default is 0.6). The width of the dendrograms' branches.
 #' If NULL then it is ignored. If the "lwd" is already defined in Rowv/Colv then this
 #' parameter is ignored (it is checked using \link[dendextend]{has_edgePar}("lwd")).
 #'
 #'
-#' @param file HTML file name to save the heatmaply into. Should be a character
-#' string ending with ".html".
-#' For example: heatmaply(x, file = "heatmaply_plot.html").
-#' This should not include a directory, only the name of the file.
-#' You can relocate the file once it is created, or use \link{setwd} first.
-#' This is based on \link[htmlwidgets]{saveWidget}.
+#' @param file name of the file(s) into which to save the heatmaply output.
+#' Should be a character vector of strings ending with ".html" for a dynamic output,
+#' or ".png", ".jpeg", ".pdf" for a static output.
+#'
+#' For example: heatmaply(x, file = "heatmaply_plot.html") or
+#' dir.create("folder")
+#' heatmaply(x, file = "folder/heatmaply_plot.html")
+#' This is based on \link[htmlwidgets]{saveWidget}, and \link[webshot]{webshot} for the static files.
+#' For more refined control over the static file output, you should save the heatmaply object using \link[webshot]{webshot}.
+#'
+#' Another example: heatmaply(x, file = c("heatmaply_plot.html", "heatmaply_plot.png"))
 #'
 #' @param long_data Data in long format. Replaces x, so both should not be used.
 #'  Colnames must be c("name", "variable", "value"). If you do not have a names
 #'  column you can simply use a sequence of numbers from 1 to the number of "rows"
 #'  inthe data.
 #'
-#' @param label_names Names for labells of x, y and value/fill mouseover.
+#' @param label_names Names for labels of x, y and value/fill mouseover.
 #' @param fontsize_row,fontsize_col,cexRow,cexCol Font size for row and column labels.
 #' @param subplot_widths,subplot_heights The relative widths and heights of each
 #'  subplot. The length of these vectors will vary depending on the number of
@@ -244,6 +250,15 @@ is.plotly <- function(x) {
 #' This option should be used when working with medium to large matrix size as it
 #' makes the heatmap much faster (and the hover still works).
 #'
+#' @param dynamicTicks (defalut: FALSE). passed to \link[plotly]{ggplotly}:
+#' should plotly.js dynamically generate axis tick labels?
+#' Dynamic ticks are useful for updating ticks in response to zoom/pan interactions; however,
+#' they can not always reproduce labels as they would appear in the static ggplot2 image.
+#'
+#' @param labRow,labCol character vectors with row and column labels to use; these default to rownames(x) or colnames(x), respectively.
+#' if set to NA, they change the value in showticklabels to be FALSE. This is mainly to keep
+#' backward compatibility with gplots::heatmap.2.
+#'
 #' @export
 #' @examples
 #' \dontrun{
@@ -293,6 +308,12 @@ is.plotly <- function(x) {
 #'
 #' # We can save heatmaply as an HTML file by using:
 #' heatmaply(iris[,-5], file = "heatmaply_iris.html")
+#' # or a png/pdf/jpeg file using:
+#' heatmaply(iris[,-5], file = "heatmaply_iris.png")
+#' # or just doing it in one go:
+#' heatmaply(iris[,-5], file = c("heatmaply_iris.html", "heatmaply_iris.png") )
+#'
+#'
 #'
 #' # If we don't want the HTML to be selfcontained, we can use the following:
 #' library(heatmaply)
@@ -476,18 +497,19 @@ heatmaply.default <- function(x,
                               key.title = NULL,
                               return_ppxpy = FALSE,
                               row_side_colors,
-                              row_side_palette,
+                              row_side_palette = NULL,
                               col_side_colors,
-                              col_side_palette,
+                              col_side_palette = NULL,
                               ColSideColors = NULL,
                               RowSideColors = NULL,
                               seriate = c("OLO", "mean", "none", "GW"),
                               heatmap_layers = NULL,
+                              side_color_layers = NULL,
                               branches_lwd = 0.6,
                               file,
                               long_data,
                               plot_method = c("ggplot", "plotly"),
-                              label_names = c("row", "column", "value"),
+                              label_names = NULL,
                               fontsize_row = 10,
                               fontsize_col = 10,
                               cexRow, cexCol,
@@ -499,6 +521,10 @@ heatmaply.default <- function(x,
                               colorbar_xpos = if(row_dend_left) -0.1 else 1.1,
                               colorbar_ypos = 0,
                               showticklabels = c(TRUE, TRUE),
+                              dynamicTicks = FALSE,
+
+                              labRow, labCol,
+
                               col) {
 
   if (!missing(long_data)) {
@@ -551,8 +577,8 @@ heatmaply.default <- function(x,
     row_side_colors <- RowSideColors
   }
 
-  if (!missing(cexRow)) fontsize_row <- cexRow
-  if (!missing(cexCol)) fontsize_col <- cexCol
+  if (!missing(cexRow)) fontsize_row <- if(is.numeric(cexRow)) cexRow*10 else cexRow
+  if (!missing(cexCol)) fontsize_col <- if(is.numeric(cexCol)) cexCol*10 else cexCol
 
   # TODO: maybe create heatmaply.data.frame heatmaply.matrix instead.
   #       But right now I am not sure this would be needed.
@@ -635,11 +661,10 @@ heatmaply.default <- function(x,
                      key.title = key.title,
                      return_ppxpy = return_ppxpy,
                      margins = margins,
-                     row_side_colors = row_side_colors,
                      row_side_palette = row_side_palette,
-                     col_side_colors = col_side_colors,
                      col_side_palette = col_side_palette,
                      heatmap_layers = heatmap_layers,
+                     side_color_layers = side_color_layers,
                      ColSideColors = ColSideColors,
                      RowSideColors = RowSideColors,
                      branches_lwd = branches_lwd,
@@ -657,17 +682,399 @@ heatmaply.default <- function(x,
                      colorbar_yanchor = colorbar_yanchor,
                      colorbar_xpos = colorbar_xpos,
                      colorbar_ypos = colorbar_ypos,
-                     showticklabels = showticklabels
+                     showticklabels = showticklabels,
+                     dynamicTicks = dynamicTicks,
+
+                     labRow = labRow, labCol = labCol
+
                      )
 
                      # TODO: think more on what should be passed in "..."
 
-  if (!missing(file)) hmly %>% saveWidget(file = file, selfcontained = TRUE)
+
+  if (!missing(file)) {
+    hmly_to_file(hmly, file)
+  }
 
   hmly
 }
 
 
+#' @export
+#' @rdname heatmaply
+heatmaply.heatmapr <- function(x,
+                               # elements for scale_fill_gradientn
+                               colors = viridis(n=256, alpha = 1, begin = 0,
+                                                    end = 1, option = "viridis"),
+                               limits = NULL,
+                               na.value = "grey50",
+                               row_text_angle = 0,
+                               column_text_angle = 45,
+                               subplot_margin = 0,
+
+                               row_dend_left = FALSE,
+                               margins = c(NA, NA, NA, NA),
+                               ...,
+                               scale_fill_gradient_fun = scale_fill_gradientn(
+                                 colors = if(is.function(colors)) colors(256) else colors,
+                                 na.value = na.value, limits = limits),
+                               grid_color = NA,
+                               grid_gap = 0,
+                               srtRow, srtCol,
+                               xlab = "", ylab = "",
+                               main = "",
+                               titleX = TRUE, titleY = TRUE,
+                               hide_colorbar = FALSE,
+                               key.title = NULL,
+                               return_ppxpy = FALSE,
+                               draw_cellnote = FALSE,
+                               cellnote_color = "auto",
+                               cellnote_textposition = "middle right",
+                               row_side_colors = x[["row_side_colors"]],
+                               row_side_palette = NULL,
+                               col_side_colors = x[["col_side_colors"]],
+                               col_side_palette = NULL,
+                               plot_method = c("ggplot", "plotly"),
+                               ColSideColors,
+                               RowSideColors,
+                               heatmap_layers = NULL,
+                               side_color_layers = NULL,
+                               branches_lwd = 0.6,
+                               label_names = c("row", "column", "value"),
+                               fontsize_row = 10,
+                               fontsize_col = 10,
+                               subplot_widths = NULL,
+                               subplot_heights = NULL,
+                               colorbar_xanchor = if(row_dend_left) "right" else "left",
+                               colorbar_yanchor = "bottom",
+                               colorbar_xpos = if(row_dend_left) -0.1 else 1.1,
+                               colorbar_ypos = 0,
+                               colorbar_len = 0.3,
+                               showticklabels = c(TRUE, TRUE),
+                               dynamicTicks = FALSE,
+
+                               labRow, labCol
+                               ) {
+
+  plot_method <- match.arg(plot_method)
+  cellnote_textposition <- match.arg(cellnote_textposition,
+    choices = c("top left", "top center" , "top right", "middle left",
+      "middle center", "middle right", "bottom left", "bottom center",
+      "bottom right"))
+
+  # informative errors for mis-specified limits
+  if (!is.null(limits)) {
+    if (!is.numeric(limits)) stop("limits must be numeric")
+    if (length(limits) != 2L) stop("limits must be of length 2 (i.e.: two dimensional)")
+
+    r <- range(as.matrix(x$matrix$data), na.rm = TRUE)
+    limits <- sort(limits)
+
+    ## Warn for broken heatmap colors
+    if (limits[1] > r[1]) {
+      limits[1] <- r[1]
+      warning("Lower limit is not <= lowest value in x, min of limits is set to the min of the range (otherwise, colors will be broken!)")
+    }
+    if (limits[2] < r[2]) {
+      limits[2] <- r[2]
+      warning("Upper limit is not >= highest value in x, max of limits is set to the max of the range (otherwise, colors will be broken!)")
+    }
+  }
+  if (!missing(srtRow)) row_text_angle <- srtRow
+  if (!missing(srtCol)) column_text_angle <- srtCol
+
+
+  if (!missing(labRow)) {
+    if(all(is.na(labRow))) {
+        showticklabels[2] <- FALSE
+      } else {
+        rownames(x$matrix$data) <- labRow
+        rownames(x$matrix$cellnote) <- labRow
+        rownames(x$cellnote) <- labRow
+        x$matrix$rows <- labRow
+      }
+    }
+  if (!missing(labCol)) {
+    if(all(is.na(labCol))) {
+      showticklabels[1] <- FALSE
+    } else {
+      colnames(x$matrix$data) <- labCol
+      colnames(x$matrix$cellnote) <- labCol
+      colnames(x$cellnote) <- labCol
+      x$matrix$cols <- labCol
+    }
+  }
+
+
+
+
+  # x is a heatmapr object.
+  # heatmapr <- list(rows = rowDend, cols = colDend, matrix = mtx, image = imgUri,
+  #                  theme = theme, options = options)
+  # x <- heatmapr(mtcars)
+  # source: http://stackoverflow.com/questions/6528180/ggplot2-plot-without-axes-legends-etc
+  theme_clear_grid_dends <- theme(axis.line=element_blank(),axis.text.x=element_blank(),
+        axis.text.y=element_blank(),axis.ticks=element_blank(),
+        axis.title.x=element_blank(),
+        axis.title.y=element_blank(), legend.position="none",
+        panel.background=element_blank(),panel.border=element_blank(),panel.grid.major=element_blank(),
+        panel.grid.minor=element_blank(),plot.background=element_blank())
+  # dendrograms:
+  rows <- x$rows
+  cols <- x$cols
+
+  if (!is.null(branches_lwd) && branches_lwd != 1) {
+    if(is.dendrogram(rows) && !has_edgePar(rows, "lwd")) rows <- set(rows, "branches_lwd", branches_lwd)
+    if(is.dendrogram(cols) && !has_edgePar(cols, "lwd")) cols <- set(cols, "branches_lwd", branches_lwd)
+  }
+
+
+
+  # this is using dendextend
+  if (is.null(cols)) {
+    py <- NULL
+  } else {
+    if (plot_method == "ggplot") {
+      col_ggdend <- as.ggdend(cols)
+      xlims <- c(0.5, nrow(col_ggdend$labels) + 0.5)
+      py <- ggplot(cols, labels  = FALSE) + theme_bw() +
+        coord_cartesian(expand = FALSE, xlim = xlims) +
+        theme_clear_grid_dends
+    } else {
+      suppressWarnings(py <- plotly_dend(cols, side = "col"))
+    }
+  }
+  if (is.null(rows)) {
+    px <- NULL
+  } else {
+    if (plot_method == "ggplot") {
+      row_ggdend <- as.ggdend(rows)
+      ylims <- c(0.5, nrow(row_ggdend$labels) + 0.5)
+
+      px <- ggplot(row_ggdend, labels  = FALSE) +
+        # coord_cartesian(expand = FALSE) +
+        coord_flip(expand = FALSE, xlim = ylims) +
+        theme_bw() +
+        theme_clear_grid_dends
+
+      if (row_dend_left) px <- px + scale_y_reverse()
+
+    } else {
+      px <- plotly_dend(rows, flip = row_dend_left, side = "row")
+    }
+  }
+  # create the heatmap
+  data_mat <- x$matrix$data
+
+  if (plot_method == "ggplot") {
+    p <- ggplot_heatmap(data_mat,
+                      row_text_angle,
+                      column_text_angle,
+                      scale_fill_gradient_fun,
+                      grid_color,
+                      key.title = key.title,
+                      layers = heatmap_layers,
+                      row_dend_left = row_dend_left,
+                      label_names = label_names,
+                      fontsize_row = fontsize_row, fontsize_col = fontsize_col)
+  } else if (plot_method == "plotly") {
+
+    p <- plotly_heatmap(data_mat, limits = limits, colors = colors,
+      key_title = key.title, label_names = label_names,
+      row_text_angle = row_text_angle, column_text_angle = column_text_angle,
+      fontsize_row = fontsize_row, fontsize_col = fontsize_col,
+      colorbar_yanchor = colorbar_yanchor, colorbar_xanchor = colorbar_xanchor,
+      colorbar_xpos = colorbar_xpos, colorbar_ypos = colorbar_ypos,
+      colorbar_len = colorbar_len)
+  }
+
+
+
+
+
+
+  # TODO: Add native plotly sidecolor function.
+  # TODO: Possibly use function to generate all 3 plots to prevent complex logic here
+  if (is.null(row_side_colors)) {
+    pr <- NULL
+  } else {
+    side_color_df <- x[["row_side_colors"]]
+    if (is.matrix(side_color_df)) side_color_df <- as.data.frame(side_color_df)
+    assert_that(
+      nrow(side_color_df) == nrow(data_mat),
+      is.data.frame(side_color_df)
+    )
+    ## Just make sure it's character first
+    side_color_df[] <- lapply(side_color_df, as.character)
+    if (plot_method == "ggplot") {
+      pr <- ggplot_side_color_plot(side_color_df,
+        type = "row",
+        text_angle = column_text_angle,
+        palette = row_side_palette,
+        fontsize = fontsize_col,
+        is_colors = !is.null(RowSideColors),
+        label_name = label_names[[1]]) + side_color_layers
+    } else {
+      pr <- plotly_side_color_plot(side_color_df,
+        type = "row",
+        text_angle = column_text_angle,
+        palette = row_side_palette,
+        fontsize = fontsize_col,
+        is_colors = !is.null(RowSideColors),
+        label_name = label_names[[1]])
+    }
+  }
+
+  if (is.null(col_side_colors)) {
+    pc <- NULL
+  } else {
+    warning("The hover text for col_side_colors is currently not implemented (due to an issue in plotly). We hope this would get resolved in future releases.")
+
+    side_color_df <- x[["col_side_colors"]]
+    if (is.matrix(side_color_df)) side_color_df <- as.data.frame(side_color_df)
+    assert_that(
+      nrow(side_color_df) == ncol(data_mat),
+      is.data.frame(side_color_df)
+    )
+    ## Just make sure it's character first
+    side_color_df[] <- lapply(side_color_df, as.character)
+    if (plot_method == "ggplot") {
+      pc <- ggplot_side_color_plot(side_color_df,
+        type = "column",
+        text_angle = row_text_angle,
+        palette = col_side_palette,
+        is_colors = !is.null(ColSideColors),
+        fontsize = fontsize_col,
+        label_name = label_names[[2]]) + side_color_layers
+    } else {
+      pc <- plotly_side_color_plot(side_color_df,
+        type = "column",
+        text_angle = row_text_angle,
+        palette = col_side_palette,
+        fontsize = fontsize_col,
+        is_colors = !is.null(ColSideColors),
+        label_name = label_names[[2]])
+    }
+  }
+
+  if (return_ppxpy) {
+    return(list(p=p, px=px, py=py, pr=pr, pc=pc))
+  } else {
+  	if (!is.null(pc)) {
+      pc <- ggplotly(pc)
+  		pc <- layout(pc, showlegend = TRUE)
+  	}
+  	if (!is.null(pr)) {
+      pr <- ggplotly(pr)
+  		pr <- layout(pr, showlegend = TRUE)
+  	}
+  }
+
+  ## plotly:
+  # turn p, px, and py to plotly objects if necessary
+  if (!is.plotly(p)) p <- ggplotly(p, dynamicTicks = dynamicTicks) %>% layout(showlegend=TRUE)
+
+
+  if (draw_cellnote) {
+    ## Predict cell color luminosity based on colorscale
+    if (cellnote_color == "auto") {
+      cellnote_color <- predict_colors(p, plot_method = plot_method)
+    }
+
+    df <- as.data.frame(x[["cellnote"]])
+    df$row <- 1:nrow(df)
+    mdf <- reshape2::melt(df, id.vars="row")
+    ## TODO: Enforce same dimnames to ensure it's not scrambled?
+    # mdf$variable <- factor(mdf$variable, levels = p$x$layout$xaxis$ticktext)
+    mdf$variable <- as.numeric(as.factor(mdf$variable))
+    mdf$value <- factor(mdf$value)
+
+    p <- p %>% add_trace(y = mdf$row, x = mdf$variable, text = mdf$value,
+        type = "scatter", mode = "text", textposition = cellnote_textposition,
+        hoverinfo = "none",
+        textfont = list(color = plotly::toRGB(cellnote_color), size = 12)
+      )
+  }
+  if (!is.null(px) && !is.plotly(px)) {
+    px <- ggplotly(px, tooltip = "y", dynamicTicks = dynamicTicks) %>%
+      layout(showlegend = FALSE)
+  }
+  if (!is.null(py) && !is.plotly(py)) {
+    py <- ggplotly(py, tooltip = "y", dynamicTicks = dynamicTicks) %>%
+      layout(showlegend = FALSE)
+  }
+
+  # https://plot.ly/r/reference/#Layout_and_layout_style_objects
+  p <- layout(p,              # all of layout's properties: /r/reference/#layout
+              title = main, # layout's title: /r/reference/#layout-title
+              xaxis = list(           # layout's xaxis is a named list. List of valid keys: /r/reference/#layout-xaxis
+                title = xlab     # xaxis's title: /r/reference/#layout-xaxis-title
+                # showgrid = T        # xaxis's showgrid: /r/reference/#layout-xaxis-showgrid
+              ),
+              yaxis = list(           # layout's yaxis is a named list. List of valid keys: /r/reference/#layout-yaxis
+                title = ylab      # yaxis's title: /r/reference/#layout-yaxis-title
+              ))
+  if (hide_colorbar) {
+    p <- hide_colorbar(p)
+    # px <- hide_colorbar(px)
+    # py <- hide_colorbar(py)
+  }
+
+  # Adjust top based on whether main is empty or not.
+  if (is.na(margins[3])) margins[3] <- ifelse(main == "", 0, 30)
+
+  rn <- c(rownames(data_mat), colnames(x[["col_side_colors"]]))
+  min_marg_row <- calc_margin(rn, fontsize = p$x$layout$yaxis$tickfont$size)
+  if (row_dend_left && is.na(margins[4])) {
+    margins[4] <- min_marg_row
+  } else if (!row_dend_left && is.na(margins[2])) {
+    margins[2] <- min_marg_row
+  }
+  if (is.na(margins[1])) {
+    cn <- c(colnames(data_mat), colnames(x[["row_side_colors"]]))
+    margins[1] <- calc_margin(cn, fontsize = p$x$layout$yaxis$tickfont$size)
+  }
+
+  # add a white grid
+  if(grid_gap > 0) {
+    p <- style(p, xgap = grid_gap, ygap = grid_gap)
+    # doesn't seem to work.
+    # if(!is.null(pr)) pr <- style(pr, xgap = grid_gap)
+    # if(!is.null(pc)) pc <- style(pc, ygap = grid_gap)
+  }
+
+
+  if(!all(showticklabels)) {
+    if(!is.logical(showticklabels)) stop("showticklabels must be a logical vector of length 2 or 1")
+    if(length(showticklabels) == 1) showticklabels <- rep(showticklabels, 2)
+    p <- p %>%
+      layout(xaxis = list(showticklabels = showticklabels[1]),
+             yaxis = list(showticklabels = showticklabels[2]))
+
+      # ggplotly() %>%
+      # layout(yaxis = list(tickmode='auto'),
+      #        xaxis = list(tickmode='auto'))
+  }
+
+
+
+  heatmap_subplot <- heatmap_subplot_from_ggplotly(p = p, px = px, py = py,
+    row_dend_left = row_dend_left, subplot_margin = subplot_margin,
+    widths = subplot_widths, heights = subplot_heights,
+    titleX = titleX, titleY = titleY, pr = pr, pc = pc, plot_method = plot_method)
+  l <- layout(heatmap_subplot,
+      margin = list(l = margins[2], b = margins[1], t = margins[3], r = margins[4]),
+      legend = list(y = 1, yanchor = "top")
+    )
+
+  # keep only relevant plotly options
+  l <- config(l, displaylogo = FALSE, collaborate = FALSE,
+        modeBarButtonsToRemove = c("sendDataToCloud", "select2d", "lasso2d","autoScale2d", "hoverClosestCartesian", "hoverCompareCartesian", "sendDataToCloud"))
+
+  l
+
+}
+
 
 
 
@@ -687,7 +1094,7 @@ heatmap_subplot_from_ggplotly <- function(p, px, py, pr, pc,
       if (is.null(pr)) {
         widths <- c(0.8, 0.2)
       } else {
-        widths <- c(0.7, 0.05, 0.2)
+        widths <- c(0.7, 0.1, 0.2)
       }
     } else {
       if (is.null(pr)) {
@@ -704,7 +1111,7 @@ heatmap_subplot_from_ggplotly <- function(p, px, py, pr, pc,
       if (is.null(pc)) {
         heights <- c(0.2, 0.8)
       } else {
-        heights <- c(0.2, 0.05, 0.7)
+        heights <- c(0.2, 0.1, 0.7)
       }
     } else {
       if (is.null(pc)) {
@@ -797,381 +1204,11 @@ heatmap_subplot_from_ggplotly <- function(p, px, py, pr, pc,
   return(s)
 }
 
-#' @export
-#' @rdname heatmaply
-heatmaply.heatmapr <- function(x,
-                               # elements for scale_fill_gradientn
-                               colors = viridis(n=256, alpha = 1, begin = 0,
-                                                    end = 1, option = "viridis"),
-                               limits = NULL,
-                               na.value = "grey50",
-                               row_text_angle = 0,
-                               column_text_angle = 45,
-                               subplot_margin = 0,
-
-                               row_dend_left = FALSE,
-                               margins = c(NA, NA, NA, NA),
-                               ...,
-                               scale_fill_gradient_fun = scale_fill_gradientn(
-                                 colors = if(is.function(colors)) colors(256) else colors,
-                                 na.value = na.value, limits = limits),
-                               grid_color = NA,
-                               grid_gap = 0,
-                               srtRow, srtCol,
-                               xlab = "", ylab = "",
-                               main = "",
-                               titleX = TRUE, titleY = TRUE,
-                               hide_colorbar = FALSE,
-                               key.title = NULL,
-                               return_ppxpy = FALSE,
-                               draw_cellnote = FALSE,
-                               cellnote_color = "auto",
-                               cellnote_textposition = "middle right",
-                               row_side_colors,
-                               row_side_palette,
-                               col_side_colors,
-                               col_side_palette,
-                               plot_method = c("ggplot", "plotly"),
-                               ColSideColors,
-                               RowSideColors,
-                               heatmap_layers = NULL,
-                               branches_lwd = 0.6,
-                               label_names,
-                               fontsize_row = 10,
-                               fontsize_col = 10,
-                               subplot_widths = NULL,
-                               subplot_heights = NULL,
-                               colorbar_xanchor = if(row_dend_left) "right" else "left",
-                               colorbar_yanchor = "bottom",
-                               colorbar_xpos = if(row_dend_left) -0.1 else 1.1,
-                               colorbar_ypos = 0,
-                               colorbar_len = 0.3,
-                               showticklabels = c(TRUE, TRUE)
-                               ) {
-
-  plot_method <- match.arg(plot_method)
-  cellnote_textposition <- match.arg(cellnote_textposition,
-    choices = c("top left", "top center" , "top right", "middle left",
-      "middle center", "middle right", "bottom left", "bottom center",
-      "bottom right"))
-
-  # informative errors for mis-specified limits
-  if (!is.null(limits)) {
-    if (!is.numeric(limits)) stop("limits must be numeric")
-    if (length(limits) != 2L) stop("limits must be of length 2 (i.e.: two dimensional)")
-
-    r <- range(as.matrix(x$matrix$data), na.rm = TRUE)
-    limits <- sort(limits)
-
-    ## Warn for broken heatmap colors
-    if (limits[1] > r[1]) {
-      limits[1] <- r[1]
-      warning("Lower limit is not <= lowest value in x, min of limits is set to the min of the range (otherwise, colors will be broken!)")
-    }
-    if (limits[2] < r[2]) {
-      limits[2] <- r[2]
-      warning("Upper limit is not >= highest value in x, max of limits is set to the max of the range (otherwise, colors will be broken!)")
-    }
-  }
-  if (!missing(srtRow)) row_text_angle <- srtRow
-  if (!missing(srtCol)) column_text_angle <- srtCol
-
-  # x is a heatmapr object.
-  # heatmapr <- list(rows = rowDend, cols = colDend, matrix = mtx, image = imgUri,
-  #                  theme = theme, options = options)
-  # x <- heatmapr(mtcars)
-  # source: http://stackoverflow.com/questions/6528180/ggplot2-plot-without-axes-legends-etc
-  theme_clear_grid_dends <- theme(axis.line=element_blank(),axis.text.x=element_blank(),
-        axis.text.y=element_blank(),axis.ticks=element_blank(),
-        axis.title.x=element_blank(),
-        axis.title.y=element_blank(),legend.position="none",
-        panel.background=element_blank(),panel.border=element_blank(),panel.grid.major=element_blank(),
-        panel.grid.minor=element_blank(),plot.background=element_blank())
-  # dendrograms:
-  rows <- x$rows
-  cols <- x$cols
-
-  if (!is.null(branches_lwd) && branches_lwd != 1) {
-    if(is.dendrogram(rows) && !has_edgePar(rows, "lwd")) rows <- set(rows, "branches_lwd", branches_lwd)
-    if(is.dendrogram(cols) && !has_edgePar(cols, "lwd")) cols <- set(cols, "branches_lwd", branches_lwd)
-  }
-
-
-
-  # this is using dendextend
-  if (is.null(cols)) {
-    py <- NULL
-  } else {
-    if (plot_method == "ggplot") {
-      col_ggdend <- as.ggdend(cols)
-      xlims <- c(0.5, nrow(col_ggdend$labels) + 0.5)
-      py <- ggplot(cols, labels  = FALSE) + theme_bw() +
-        coord_cartesian(expand = FALSE, xlim = xlims) +
-        theme_clear_grid_dends
-    } else {
-      suppressWarnings(      py <- plotly_dend(cols, side = "col"))
-    }
-  }
-  if (is.null(rows)) {
-    px <- NULL
-  } else {
-    if (plot_method == "ggplot") {
-      row_ggdend <- as.ggdend(rows)
-      ylims <- c(0.5, nrow(row_ggdend$labels) + 0.5)
-
-      px <- ggplot(row_ggdend, labels  = FALSE) +
-        # coord_cartesian(expand = FALSE) +
-        coord_flip(expand = FALSE, xlim = ylims) +
-        theme_bw() +
-        theme_clear_grid_dends
-      if (row_dend_left) px <- px + scale_y_reverse()
-    } else {
-      px <- plotly_dend(rows, flip = row_dend_left, side = "row")
-    }
-  }
-  # create the heatmap
-  data_mat <- x$matrix$data
-
-  if (plot_method == "ggplot") {
-    p <- ggplot_heatmap(data_mat,
-                      row_text_angle,
-                      column_text_angle,
-                      scale_fill_gradient_fun,
-                      grid_color,
-                      key.title = key.title,
-                      layers = heatmap_layers,
-                      row_dend_left = row_dend_left,
-                      label_names = label_names,
-                      fontsize_row = fontsize_row, fontsize_col = fontsize_col)
-  } else if (plot_method == "plotly") {
-
-    p <- plotly_heatmap(data_mat, limits = limits, colors = colors,
-      key_title = key.title,
-      row_text_angle = row_text_angle, column_text_angle = column_text_angle,
-      fontsize_row = fontsize_row, fontsize_col = fontsize_col,
-      colorbar_yanchor = colorbar_yanchor, colorbar_xanchor = colorbar_xanchor,
-      colorbar_xpos = colorbar_xpos, colorbar_ypos = colorbar_ypos,
-      colorbar_len = colorbar_len)
-  }
-
-
-
-
-
-
-  # TODO: Add native plotly sidecolor function.
-  # TODO: Possibly use function to generate all 3 plots to prevent complex logic here
-  if (missing(row_side_colors)) {
-    pr <- NULL
-  } else {
-    side_color_df <- x[["row_side_colors"]]
-    if (is.matrix(side_color_df)) side_color_df <- as.data.frame(side_color_df)
-    assert_that(
-      nrow(side_color_df) == nrow(data_mat),
-      is.data.frame(side_color_df)
-    )
-    pr <- side_color_plot(x[["row_side_colors"]], type = "row",
-      text_angle = column_text_angle,
-      palette = row_side_palette,
-      is_colors = !is.null(RowSideColors), label_name = label_names[[1]])
-  }
-
-  if (missing(col_side_colors)) {
-    pc <- NULL
-  } else {
-    warning("The hover text for col_side_colors is currently not implemented (due to an issue in plotly). We hope this would get resolved in future releases.")
-
-    side_color_df <- x[["col_side_colors"]]
-    if (is.matrix(side_color_df)) side_color_df <- as.data.frame(side_color_df)
-    assert_that(
-      nrow(side_color_df) == ncol(data_mat),
-      is.data.frame(side_color_df)
-    )
-    ## Just make sure it's character first
-    side_color_df[] <- lapply(side_color_df, as.character)
-    pc <- side_color_plot(side_color_df, type = "column",
-      text_angle = row_text_angle,
-      palette = col_side_palette,
-      is_colors = !is.null(ColSideColors),
-      label_name = label_names[[2]]
-    )
-  }
-
-  if (return_ppxpy) {
-    return(list(p=p, px=px, py=py, pr=pr, pc=pc))
-  }
-
-  ## plotly:
-  # turn p, px, and py to plotly objects if necessary
-  if (!is.plotly(p)) p <- ggplotly(p) %>% layout(showlegend=FALSE)
-
-  if (draw_cellnote) {
-    ## Predict cell color luminosity based on colorscale
-    if (cellnote_color == "auto") {
-      cellnote_color <- predict_colors(p, plot_method)
-    }
-
-    df <- as.data.frame(x[["cellnote"]])
-    df$row <- 1:nrow(df)
-    mdf <- reshape2::melt(df, id.vars="row")
-    ## TODO: Enforce same dimnames to ensure it's not scrambled?
-    # mdf$variable <- factor(mdf$variable, levels = p$x$layout$xaxis$ticktext)
-    mdf$variable <- as.numeric(as.factor(mdf$variable))
-    mdf$value <- factor(mdf$value)
-
-    p <- p %>% add_trace(y = mdf$row, x = mdf$variable, text = mdf$value,
-        type = "scatter", mode = "text", textposition = cellnote_textposition,
-        hoverinfo = "none",
-        textfont = list(color = plotly::toRGB(cellnote_color), size = 12)
-      )
-  }
-  if (!is.null(px) && !is.plotly(px)) {
-    px <- ggplotly(px, tooltip = "y") %>%
-      layout(showlegend=FALSE)
-  }
-  if (!is.null(py) && !is.plotly(py)) {
-    py <- ggplotly(py, tooltip = "y") %>%
-      layout(showlegend=FALSE)
-  }
-
-  # https://plot.ly/r/reference/#Layout_and_layout_style_objects
-  p <- layout(p,              # all of layout's properties: /r/reference/#layout
-              title = main, # layout's title: /r/reference/#layout-title
-              xaxis = list(           # layout's xaxis is a named list. List of valid keys: /r/reference/#layout-xaxis
-                title = xlab     # xaxis's title: /r/reference/#layout-xaxis-title
-                # showgrid = T        # xaxis's showgrid: /r/reference/#layout-xaxis-showgrid
-              ),
-              yaxis = list(           # layout's yaxis is a named list. List of valid keys: /r/reference/#layout-yaxis
-                title = ylab      # yaxis's title: /r/reference/#layout-yaxis-title
-              ))
-  if (hide_colorbar) {
-    p <- hide_colorbar(p)
-    # px <- hide_colorbar(px)
-    # py <- hide_colorbar(py)
-  }
-
-  # Adjust top based on whether main is empty or not.
-  if (is.na(margins[3])) margins[3] <- ifelse(main == "", 0, 30)
-
-
-  min_marg_row <- calc_margin(rownames(data_mat),
-    fontsize = p$x$layout$yaxis$tickfont$size)
-  if (row_dend_left && is.na(margins[4])) {
-    margins[4] <- min_marg_row
-  } else if (!row_dend_left && is.na(margins[2])) {
-    margins[2] <- min_marg_row
-  }
-  if (is.na(margins[1])) {
-    margins[1] <- calc_margin(colnames(data_mat),
-        fontsize = p$x$layout$yaxis$tickfont$size)
-  }
-
-  # add a white grid
-  if(grid_gap > 0) {
-    p <- style(p, xgap = grid_gap, ygap = grid_gap)
-    # doesn't seem to work.
-    # if(!is.null(pr)) pr <- style(pr, xgap = grid_gap)
-    # if(!is.null(pc)) pc <- style(pc, ygap = grid_gap)
-  }
-
-
-  if(!all(showticklabels)) {
-    if(!is.logical(showticklabels)) stop("showticklabels must be a logical vector of length 2 or 1")
-    if(length(showticklabels) == 1) showticklabels <- rep(showticklabels, 2)
-    p <- p %>%
-      layout(xaxis = list(showticklabels = showticklabels[1]),
-             yaxis = list(showticklabels = showticklabels[2]))
-
-      # ggplotly() %>%
-      # layout(yaxis = list(tickmode='auto'),
-      #        xaxis = list(tickmode='auto'))
-  }
-
-
-
-  heatmap_subplot <- heatmap_subplot_from_ggplotly(p = p, px = px, py = py,
-    row_dend_left = row_dend_left, subplot_margin = subplot_margin,
-    widths = subplot_widths, heights = subplot_heights,
-    titleX = titleX, titleY = titleY, pr = pr, pc = pc, plot_method = plot_method)
-  l <- layout(heatmap_subplot,
-      margin = list(l = margins[2], b = margins[1], t = margins[3], r = margins[4]),
-      legend = list(y=1, yanchor="top")
-    )
-
-  # keep only relevant plotly options
-  l <- config(l, displaylogo = FALSE, collaborate = FALSE,
-        modeBarButtonsToRemove = c("sendDataToCloud", "select2d", "lasso2d","autoScale2d", "hoverClosestCartesian", "hoverCompareCartesian", "sendDataToCloud"))
-
-  l
-
-}
 
 ## TODO: Better/safer estimation of total size, or use monospace.
 calc_margin <- function(labels, fontsize) {
     max(nchar(labels) * fontsize, na.rm = TRUE) * 0.6
   # http://stackoverflow.com/questions/19113725/what-dependency-between-font-size-and-width-of-char-in-monospace-font
-
 }
 
 
-
-
-## Predict luminosity of cells and change text based on that
-predict_colors <- function(p, plot_method) {
-
-  ## http://stackoverflow.com/questions/12043187/how-to-check-if-hex-color-is-too-black
-  colorscale_df <- p$x$data[[1]]$colorscale
-  cell_values <- as.data.frame(p$x$data[[1]]$z)
-  cell_values$row <- 1:nrow(cell_values)
-  cell_values_m <- reshape2::melt(cell_values, id.vars = "row")
-  cell_values_vector <- cell_values_m$value
-  ## Need to normalise to (0, 1) scale as this is what plotly
-  ## uses internally
-  if (plot_method == "plotly") {
-    ## Need to convert plotly colors to hex colors
-    colorscale_df[, 2] <- parse_plotly_color(colorscale_df[, 2])
-
-    cell_values_vector <- normalize(as.numeric(cell_values_vector))
-    ## interpolate to 256 colors because that's probably enough
-    colorscale_df <- data.frame(
-      stats::approx(as.numeric(colorscale_df[, 1]), n = 256)$y,
-      grDevices::colorRampPalette(colorscale_df[, 2])(256)
-    )
-
-    ## Then need to sort, find nearest neighbour, and map across
-    cell_values_vector_sort <- sort(cell_values_vector)
-    nearest_neighbours <- sapply(cell_values_vector_sort,
-      function(val) {
-        max(colorscale_df[as.numeric(colorscale_df[, 1]) <= val, 1])
-      }
-    )
-    names(nearest_neighbours) <- cell_values_vector_sort
-    cell_values_vector <- nearest_neighbours[as.character(cell_values_vector)]
-  }
-
-  cell_values_vector <- as.character(cell_values_vector)
-  ind <- match(cell_values_vector, colorscale_df[, 1])
-  cell_colors <- unlist(colorscale_df[ind, 2])
-  cell_colors_rgb <- colorspace::hex2RGB(cell_colors)
-  cell_font_colors <- sapply(seq_len(nrow(cell_colors_rgb@coords)),
-    function(i) {
-      col <- cell_colors_rgb@coords[i, ]
-      luma <- (0.2126 * col[1]) +
-        (0.7152 * col[2]) +
-        (0.0722 * col[3])
-      ifelse (luma < 0.4, "white", "black")
-    }
-  )
-  cell_font_colors
-}
-
-
-parse_plotly_color <- function(color) {
-  r <- gsub("rgb[a]?\\((\\d+),(\\d+),(\\d+),\\d+)",
-    "\\1", color)
-  g <- gsub("rgb[a]?\\((\\d+),(\\d+),(\\d+),\\d+)",
-    "\\2", color)
-  b <- gsub("rgb[a]?\\((\\d+),(\\d+),(\\d+),\\d+)",
-    "\\3", color)
-  rgb(r, g, b, maxColorValue = 255)
-}
